﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;

namespace howto_crop_to_size
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private Rectangle SelectedRect = new Rectangle();
        private bool Dragging = false;
        private Point LastPoint;

        // Set the selection rectangle's initial size.
        private void Form1_Load(object sender, EventArgs e)
        {
            int width = int.Parse(txtWidth.Text);
            int height = int.Parse(txtHeight.Text);
            SelectedRect = new Rectangle(0, 0, width, height);
        }

        // Let the user select an image file.
        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Load the image.
                    picImage.Image = new Bitmap(ofdImage.FileName);
                    picImage.Visible = true;
                    picImage.Refresh();

                    // Make the form bigger if necessary.
                    int width = picImage.Right + picImage.Left;
                    int height = picImage.Bottom + picImage.Left;
                    if (width < ClientSize.Width) width = ClientSize.Width;
                    if (height < ClientSize.Height) height = ClientSize.Height;
                    ClientSize = new Size(width, height);

                    // Enable the Save As menu item.
                    mnuFileSaveAs.Enabled = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        // Save the selected area.
        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Copy the selected area into a new Bitmap.
                    Bitmap bm = new Bitmap(
                        SelectedRect.Width,
                        SelectedRect.Height);
                    using (Graphics gr = Graphics.FromImage(bm))
                    {
                        gr.DrawImage(picImage.Image, 0, 0,
                            SelectedRect, GraphicsUnit.Pixel);
                    }

                    // Save the new Bitmap.
                    SaveImage(bm, sfdImage.FileName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        // Draw the selection rectangle.
        private void picImage_Paint(object sender, PaintEventArgs e)
        {
            try
            {
                // Draw the selection rectangle.
                using (Pen pen = new Pen(Color.Red, 2))
                {
                    e.Graphics.DrawRectangle(pen, SelectedRect);

                    pen.Color = Color.Yellow;
                    pen.DashPattern = new float[] { 5, 5 };
                    e.Graphics.DrawRectangle(pen, SelectedRect);
                }
            }
            catch
            {
            }
        }

        // Return the currently selected area.
        private void UpdateSelection()
        {
            int width, height;

            // Get the desired dimensions.
            // If there's a problem, return a zero-size rectangle.
            if (!int.TryParse(txtWidth.Text, out width)) return;
            if (!int.TryParse(txtHeight.Text, out height)) return;

            // Return the rectangle.
            SelectedRect.Width = width;
            SelectedRect.Height = height;
        }

        // Start dragging.
        private void picImage_MouseDown(object sender, MouseEventArgs e)
        {
            // If the mouse is not inside the
            // selection rectangle, do nothing.
            if (!SelectedRect.Contains(e.Location)) return;

            // Start the drag.
            LastPoint = e.Location;
            Dragging = true;
        }

        // Continue dragging.
        private void picImage_MouseMove(object sender, MouseEventArgs e)
        {
            // See if we are dragging.
            if (Dragging)
            {
                // We are dragging. Move the selected rectangle.
                SelectedRect.X += e.Location.X - LastPoint.X;
                SelectedRect.Y += e.Location.Y - LastPoint.Y;
                LastPoint = e.Location;
                picImage.Refresh();
            }
            else
            {
                // We are not dragging. Update the cursor.
                if (SelectedRect.Contains(e.Location))
                    Cursor = Cursors.SizeAll;
                else
                    Cursor = Cursors.Default;
            }
        }

        // Stop dragging.
        private void picImage_MouseUp(object sender, MouseEventArgs e)
        {
            Dragging = false;
        }

        private void txtWidth_TextChanged(object sender, EventArgs e)
        {
            // Get the new value.
            int value;
            if (!int.TryParse(txtWidth.Text, out value)) value = 0;
            SelectedRect.Width = value;
            picImage.Refresh();
        }

        private void txtHeight_TextChanged(object sender, EventArgs e)
        {
            int value;
            if (!int.TryParse(txtHeight.Text, out value)) value = 0;
            SelectedRect.Height = value;
            picImage.Refresh();
        }
    }
}
